import pandas as pd
import numpy as np
import pytest
from sklearn.dummy import DummyRegressor

from moosir_feature.model_validations.basic_parameter_searcher import ParameterSearcher


def test_run_parameter_search():
    # arrange
    sample_n = 100
    feature_n = 1

    feature_cols = [f"f_{i}" for i in range(feature_n)]
    target_col = "target"
    features = pd.DataFrame(data=np.arange(0, sample_n * feature_n).reshape(sample_n, -1), columns=feature_cols)
    targets = pd.DataFrame(data=np.arange(0, sample_n * 1).reshape(sample_n, -1), columns=[target_col])

    # arrange params
    cv_params = {"train_lengths": [6],
                 "test_lengths": [3],
                 "look_aheads": [1],
                 "train_shuffle_block_size": [2]}

    estimator = DummyRegressor(strategy="mean")

    param_grid = {"strategy": ["mean", "median"]}

    metrics = ["neg_mean_squared_error"]

    searcher = ParameterSearcher()

    # act
    grid_results = searcher.run_parameter_search_multiple_cvs(X=features,
                                                              y=targets,
                                                              estimator=estimator,
                                                              cv_params=cv_params,
                                                              param_grid=param_grid,
                                                              metrics=metrics,
                                                              )

    # assert
    print(grid_results)
    # print(best_models)


def test_search_params():
    # arrange
    sample_n = 100
    feature_n = 1
    train_n = 70
    test_n = 25

    feature_cols = [f"f_{i}" for i in range(feature_n)]
    target_col = "target"
    features = pd.DataFrame(data=np.arange(0, sample_n * feature_n).reshape(sample_n, -1), columns=feature_cols)
    targets = pd.DataFrame(data=np.arange(0, sample_n * 1).reshape(sample_n, -1), columns=[target_col])

    # arrange params
    estimator = DummyRegressor(strategy="mean")
    param_grid = {"strategy": ["mean", "median"]}
    metrics = ["neg_mean_squared_error"]

    searcher = ParameterSearcher()

    # act
    cv_result, grid = searcher.search_params(X=features,
                                             y=targets,
                                             estimator=estimator,
                                             param_grid=param_grid,
                                             metrics=metrics,
                                             train_size=train_n,
                                             test_size=test_n,
                                             is_get_best_model=False,
                                             return_train_score=True,
                                             train_shuffle_block_size=None)

    # assert
    print(cv_result)
    print(grid)



from sklearn.model_selection import cross_val_score, GridSearchCV, RandomizedSearchCV
import pandas as pd
import statsmodels.api as sm

from itertools import product

from .model_validator import CustomTsCv


class ParameterSearcher:
    """
        - todo: result in here will be used in research/reporting/shared, bad design, need to move to common!!!!
        - todo: copied from prob models but made some changes: bad design, need to be in common
        - todo: how to add custom metrics, such as bias and variance
        - todo: does not support multiple metrics cos of best model!!!
        - todo: doesnt have unit tests
    """

    def __init__(self):
        self.train_lengths_key = "train_lengths"
        self.test_lengths_key = "test_lengths"
        self.look_aheads_key = "look_aheads"
        self.train_shuffle_block_size = "train_shuffle_block_size"

    def search_params(self,
                      X: pd.DataFrame,
                      y: pd.DataFrame,
                      train_size: int,
                      test_size: int,
                      train_shuffle_block_size: int,
                      estimator,
                      param_grid: dict,
                      metrics,
                      is_get_best_model: bool,
                      return_train_score:bool):

        refit = False
        scoring = metrics
        if is_get_best_model:
            assert len(metrics) == 1
            scoring = metrics[0]
            refit = True

        obs_n = X.shape[0]

        cv = CustomTsCv(train_n=train_size,
                        test_n=test_size,
                        sample_n=obs_n,
                        train_shuffle_block_size=train_shuffle_block_size)

        grid = RandomizedSearchCV(estimator=estimator,
                                  param_distributions=param_grid,
                                  scoring=scoring,
                                  verbose=1,
                                  # n_jobs=-1,
                                  cv=cv,
                                  refit=refit,
                                  return_train_score=return_train_score)

        grid_result = grid.fit(X, y)

        cv_result = pd.DataFrame(grid_result.cv_results_)
        return cv_result, grid

    def run_parameter_search_multiple_cvs(self,
                                          X: pd.DataFrame,
                                          y: pd.DataFrame,
                                          estimator,
                                          param_grid,
                                          cv_params,
                                          metrics
                                          ):
        """
        :param X: x to pass to estimator
        :param y: target values
        :param estimator: estimator object. A object of that type is instantiated for each grid point
        :param param_grid: parameter dictionary to be passed to grid search
        :param metric: estimator One metrics (cos need to know best_estimator)
        :param cv_params: required for cross-validation,
                        - dictionary with "train_lengths", "test_lengths", "look_aheads" keys and values of arrays
                        - it searches for different train/test/lookaheads lengths
        :return: gridsearch cv_result + 3 columns for cv
        """

        grid_results = []
        # best_models = []

        cv_params_grid = list(product(cv_params[self.train_lengths_key],
                                      cv_params[self.test_lengths_key],
                                      cv_params[self.look_aheads_key],
                                      cv_params[self.train_shuffle_block_size]
                                      ))

        # todo: wrong, no need for loop!!!!
        for train_size, test_size, look_ahead, train_shuffle_block_size in cv_params_grid:
            result, grid = self.search_params(X=X,
                                              y=y,
                                              train_size=train_size,
                                              test_size=test_size,
                                              train_shuffle_block_size=train_shuffle_block_size,
                                              # obs_n=obs_n,
                                              estimator=estimator,
                                              param_grid=param_grid,
                                              metrics=metrics,
                                              is_get_best_model=False,
                                              return_train_score=True
                                              )

            result[self.train_lengths_key] = train_size
            result[self.test_lengths_key] = test_size
            result[self.look_aheads_key] = look_ahead

            grid_results.append(result)

            # best_model = grid.best_estimator_
            # best_models.append(best_model)

        grid_results = pd.concat(grid_results).reset_index()
        _ = grid_results.pop("params")
        _ = grid_results.pop("index")

        # indx_max = grid_results["mean_test_score"].idxmax()
        # best_model_result = best_models[indx_max]

        return grid_results #, best_models



import pandas as pd
import pytest

from moosir_feature.transformers.indicators.feature_manager import *
from moosir_feature.transformers.indicators.contexts import *




@pytest.fixture
def ohlc():
    return pd.DataFrame(data=np.random.rand(100, 4), columns=["Open", "High", "Low", "Close"])


"""
todo: test context!!!!
"""


def test_feature_manager_calculate_forwards(ohlc: pd.DataFrame):
    # arrange
    win_lens = [3, 6]
    forward_lens = [2, 5]


    # feature_names = OPERATORS_IND.keys()
    feature_names = [
        RsiOperator.__name__,
    ]

    context = Context()

    # act
    result = calculate_forwards(ohlc=ohlc,
                                win_lens=win_lens,
                                forward_periods=forward_lens,
                                feature_names=feature_names)

    # assert
    print(result)
    assert context.validate(result, operators=[RsiOperator], periods=win_lens, forward_periods=forward_lens)
    # assert len(result) == len(ohlc) - 4, "nan values generated by window still exist"



def test_feature_manager_calculate_features_lags(ohlc: pd.DataFrame):
    # arrange
    win_lens = [3, 6]
    lag_lens = [2, 5]

    feature_names = [
        RsiOperator.__name__,
    ]

    context = Context()

    # act
    result = calculate_lags(ohlc=ohlc,
                            win_lens=win_lens,
                            lag_lens=lag_lens,
                            feature_names=feature_names)

    # assert
    print(result)
    assert context.validate(result, operators=[RsiOperator], periods=win_lens, lag_periods=lag_lens)
    # assert len(result) == len(ohlc) - 4, "nan values generated by window still exist"




def test_feature_manager_calculate_features_all_indicators(ohlc: pd.DataFrame):
    # arrange
    win_lens = [3, 4]

    feature_names = OPERATORS_IND.keys()

    context = Context()

    # act
    result = calculate_features(ohlc=ohlc,
                                win_lens=win_lens,
                                feature_names=feature_names)

    # assert
    assert context.validate(result, operators=OPERATORS_IND.values(), periods=win_lens)
    assert len(result) == len(ohlc) - 4, "nan values generated by window still exist"



def test_feature_manager_calculate_features_e2e(ohlc):
    # arrange
    look_back_periods = [3, 4]

    tech_indicators = list(OPERATORS_IND.keys()) + list(OPERATORS_RET.keys())

    context = Context()

    # act
    result = calculate_features(ohlc=ohlc,
                                win_lens=look_back_periods,
                                feature_names=tech_indicators)

    # assert
    assert context.validate(result, operators=OPERATORS_IND.values(), periods=look_back_periods)
    assert context.validate(result, operators=OPERATORS_RET.values(), periods=look_back_periods)





import pandas as pd
import numpy as np
from sklearn.model_selection import cross_val_predict
from .model_validator import CustomTsCv

def predict_on_cv(model,
                  features: pd.DataFrame,
                  targets: pd.DataFrame,
                  cv: CustomTsCv) -> pd.DataFrame:
    """
      - it is to fit and predict the model on all cv test section

      - input cv needs to make all datapoint one time and only one time as a test
        - otherwise exception: "cross_val_predict only works for partitions TimeSeriesSplit"

      - returns
        - predictions for all targets index/rows
        index  pred_<target_column_name>
        ----- --------------------------
    """
    assert len(features) == len(targets), "features and targets not the same size"
    assert len(targets.columns) == 1, "targets with multiple columns not supported"

    assert cv.train_shuffle_block_size is None, "predict results distorted when shuffle indexes on split"
    result = cross_val_predict(model, features, targets, cv=cv)

    predictions = targets.copy()
    predictions[predictions.columns[0]] = result
    predictions.columns = [f"pred_{c}" for c in predictions.columns]

    return predictions


import numpy as np
import pandas as pd
from pandas.util.testing import assert_frame_equal

from moosir_feature.transformers.feature_cleaning.feature_cleaner import *

import pytest


def test_normalize_should_scale_cols_separately():
    # arrange
    n_samples = 15
    min_value_f1 = -10
    max_value_f1 = -5

    min_value_f2 = 60000
    max_value_f2 = 70000

    data = {
        "feature_1": np.random.randint(low=min_value_f1, high=max_value_f1, size=n_samples),
        "feature_2": np.random.randint(low=min_value_f2, high=max_value_f2, size=n_samples)
    }

    data = pd.DataFrame(data=data)

    # act
    transformed, scaler = normalize(data=data)

    # assert
    dec_points = 5
    assert round(transformed["feature_1"].max(), dec_points) == 1
    assert round(transformed["feature_1"].min(), dec_points) == 0

    assert round(transformed["feature_2"].max(), dec_points) == 1
    assert round(transformed["feature_2"].min(), dec_points) == 0


def test_timestamp_ohlc_resample_should_work():
    # assrange
    open = [10, 20, 30, 40]
    high = [100, 200, 300, 400]
    low = [1, 2, 3, 4]
    close = [50, 60, 70, 80]
    inds = pd.date_range(start="2022/04/01", periods=4, freq="T")

    ohlc = pd.DataFrame({"Open": open, "High": high, "Low": low, "Close": close}, index=inds)
    ohlc.index.name = "Timestamp"

    # act
    result = timestamp_ohlc_resample(ohlc=ohlc, resample_freq="2T")

    # assert
    exp_inds = pd.date_range(start="2022/04/01", periods=2, freq="2T")
    expected = pd.DataFrame(data={"Open": [10, 30], "High": [200, 400], "Low": [1, 3], "Close": [60, 80]},
                            index=exp_inds)
    expected.index.name = "Timestamp"

    assert_frame_equal(result, expected)


def test_timestamp_ohlc_resample_should_consider_uneven_highs_lows():
    # assrange
    open = [10, 20, 30, 40]
    high = [200, 100, 300, 400]
    low = [2, 1, 3, 4]
    close = [50, 60, 70, 80]
    inds = pd.date_range(start="2022/04/01", periods=4, freq="T")

    ohlc = pd.DataFrame({"Open": open, "High": high, "Low": low, "Close": close}, index=inds)
    ohlc.index.name = "Timestamp"

    # act
    result = timestamp_ohlc_resample(ohlc=ohlc, resample_freq="2T")

    # assert
    exp_inds = pd.date_range(start="2022/04/01", periods=2, freq="2T")
    expected = pd.DataFrame(data={"Open": [10, 30], "High": [200, 400], "Low": [1, 3], "Close": [60, 80]},
                            index=exp_inds)
    expected.index.name = "Timestamp"

    assert_frame_equal(result, expected)


def test_timestamp_ohlc_resample_should_return_input_when_freq_bigger():
    # assrange
    sample_n = 5
    open = np.random.randint(0, 2, size=sample_n)
    high = np.random.randint(0, 2, size=sample_n)
    low = np.random.randint(0, 2, size=sample_n)
    close = np.random.randint(0, 2, size=sample_n)

    inds = pd.date_range(start="2022/04/01", periods=sample_n)

    ohlc = pd.DataFrame({"Open": open, "High": high, "Low": low, "Close": close}, index=inds)
    ohlc.index.name = "Timestamp"

    # act
    result = timestamp_ohlc_resample(ohlc=ohlc, resample_freq="10T")

    # assert

    assert_frame_equal(result, ohlc)


def test_timestamp_ohlc_resample_input_no_freq():
    # arrange
    sample_n = 4
    open = np.random.randint(0, 2, size=sample_n)
    high = np.random.randint(0, 2, size=sample_n)
    low = np.random.randint(0, 2, size=sample_n)
    close = np.random.randint(0, 2, size=sample_n)

    # missing dates results in None Freq
    inds = pd.date_range(start="2022/04/01", periods=2).append(pd.date_range(start="2022/10/01", periods=2))

    ohlc = pd.DataFrame({"Open": open, "High": high, "Low": low, "Close": close}, index=inds)
    ohlc.index.name = "Timestamp"

    # act
    result = timestamp_ohlc_resample(ohlc=ohlc, resample_freq="2T")

    # assert
    assert_frame_equal(result, ohlc)


def test_timestamp_ohlc_resample_input_no_freq_bigger_requested_freq():
    # arrange
    sample_n = 4
    open = np.random.randint(0, 2, size=sample_n)
    high = np.random.randint(0, 2, size=sample_n)
    low = np.random.randint(0, 2, size=sample_n)
    close = np.random.randint(0, 2, size=sample_n)

    # missing dates results in None Freq
    inds = pd.date_range(start="2022/04/01", periods=2, freq="T").append(
        pd.date_range(start="2022/04/02", periods=2, freq="T"))

    ohlc = pd.DataFrame({"Open": open, "High": high, "Low": low, "Close": close}, index=inds)
    ohlc.index.name = "Timestamp"

    # act
    result = timestamp_ohlc_resample(ohlc=ohlc, resample_freq="2T")

    # assert
    assert len(result) == 2
    assert result.loc["2022-04-01"] is not None
    assert result.loc["2022-04-02"] is not None




import pandas as pd

import moosir_feature.transformers.feature_cleaning.feature_cleaner as f_cleaner
import moosir_feature.transformers.tsfresh_features.feature_manager as f_tsfresh
import moosir_feature.transformers.basic_features.feature_calculator as f_basic
import moosir_feature.transformers.indicators.feature_manager as f_indicator
import moosir_feature.transformers.feature_selections.feature_selector as f_selector
import moosir_feature.transformers.features_common.calculator as f_common

from .settings import *


def _create_targets(clean_data: pd.DataFrame,
                    target_settings: TargetSettings
                    ):
    if type(target_settings).__name__ == TsfreshTargetSettings.__name__:
        target_eng = f_tsfresh.calculate_forwards(ohlc=clean_data,
                                                  win_lens=[target_settings.win_len],
                                                  feature_names=[target_settings.feature_name],
                                                  forward_lens=[target_settings.lookahead_len])

        target_col = f_tsfresh.get_feature_col_by_subtype_name(feature_name=target_settings.feature_name,
                                                               win_len=target_settings.win_len,
                                                               subtype_name=target_settings.target_col_feature_subtype,
                                                               is_fwd=True,
                                                               fwd_len=target_settings.lookahead_len)

    elif type(target_settings).__name__ == IndicatorTargetSettings.__name__:
        target_eng = f_indicator.calculate_forwards(ohlc=clean_data,
                                                    win_lens=[target_settings.win_len],
                                                    feature_names=[target_settings.feature_name],
                                                    forward_periods=[target_settings.lookahead_len])

        target_col = f_indicator.get_feature_col_by_subtype_name(
            feature_name=target_settings.target_col_feature_subtype,
            columns=target_eng.columns, is_fwd=True)

    target_eng = target_eng[[target_col]]

    target_eng = f_cleaner.clip_quantile(target_eng)
    return target_eng


def _create_features(clean_data: pd.DataFrame, feature_settings: FeatureSettings):
    features_eng = clean_data.copy()
    if type(feature_settings).__name__ == TsfreshFeatureSettings.__name__:
        features_eng = f_tsfresh.calculate_features(ohlc=features_eng, win_lens=feature_settings.win_lens,
                                                    feature_names=feature_settings.feature_names)
    elif type(feature_settings).__name__ == IndicatorFeatureSettings.__name__:
        features_eng = f_indicator.calculate_features(ohlc=features_eng, win_lens=feature_settings.win_lens,
                                                      feature_names=feature_settings.feature_names)
    elif type(feature_settings).__name__ == BasicFeatureSettings.__name__:
        features_eng = f_basic.calculate_features(ohlc=features_eng,
                                                  feature_names=feature_settings.feature_names)
    else:
        raise Exception(f"feature calculator is not implemented: {type(feature_settings).__name__}")

    features_eng = f_cleaner.clip_quantile(features_eng)
    features_eng = f_common.remove_ohlc_columns_if_exists(data=features_eng)
    features = features_eng.copy()

    return features


def _create_lags(ohlc: pd.DataFrame,
                 lag_settings: LagSettings
                 ):
    if type(lag_settings).__name__ == TsfreshLagSettings.__name__:
        feature_lags = f_tsfresh.calculate_lags(ohlc=ohlc, win_lens=lag_settings.win_lens,
                                                lag_lens=lag_settings.lag_lens,
                                                feature_names=lag_settings.feature_names)
    elif type(lag_settings).__name__ == IndicatorLagSettings.__name__:
        feature_lags = f_indicator.calculate_lags(ohlc=ohlc, win_lens=lag_settings.win_lens,
                                                  lag_lens=lag_settings.lag_lens,
                                                  feature_names=lag_settings.feature_names)

    return feature_lags


def _create_features_list(data: pd.DataFrame, feature_settings_list: list):
    features_temps = []
    for f_settings in feature_settings_list:
        features = _create_features(clean_data=data.copy(),
                                    feature_settings=f_settings)
        features_temps.append(features)
    results = f_common.combine_features(features_list=features_temps)

    return results


def _create_lags_list(data: pd.DataFrame, lag_settings_list: list):
    features_temps = []
    for f_settings in lag_settings_list:
        features = _create_lags(ohlc=data.copy(),
                                lag_settings=f_settings)
        features_temps.append(features)
    results = f_common.combine_features(features_list=features_temps)

    return results


class FeatureSelectorManager:
    def __init__(self, feature_selector_settings: FeatureSelectorSettings):
        self.feature_selector_settings = feature_selector_settings

    def select_features(self, features: pd.DataFrame, targets: pd.DataFrame):
        features = f_selector.remove_low_variance_features(features=features)

        assert len(
            features.columns) > 1, f"number of features needs to be more than 1. features n: {len(features.columns)}"
        n_features_to_select = max(2,
                                   int(len(features.columns) * self.feature_selector_settings.features_to_select_perc))

        features = f_selector.remove_features_recursively(features=features, targets=targets,
                                                          n_features_to_select=n_features_to_select)

        return features, targets


def _remove_target_cols(feature_settings_list: FeatureSettings, target_col: str):
    for feature_settings in feature_settings_list:
        f_names = []
        for f in feature_settings.feature_names:
            if f != target_col:
                f_names.append(f)
        feature_settings.feature_names = f_names


class FeatureCreatorManager:
    def __init__(self,
                 target_settings: TargetSettings,
                 feature_settings_list: list,
                 lag_settings_list: list
                 ):
        # basic
        self.resample_freq = "5T"

        # targets
        self.target_settings = target_settings

        # features
        self.feature_settings_list = feature_settings_list

        # lag and lookahead
        self.lag_settings_list = lag_settings_list



    def create_features_and_targets(self, instances: pd.DataFrame):
        data = f_cleaner.apply_basic_cleaning(instances=instances, resample_freq=self.resample_freq)



        targets = _create_targets(clean_data=data.copy(),
                                  target_settings=self.target_settings)

        features = _create_features_list(data=data.copy(),
                                         feature_settings_list=self.feature_settings_list)

        features_lags = _create_lags_list(data=data.copy(),
                                          lag_settings_list=self.lag_settings_list)

        # todo: needs to be configurable otherwise close will be gone
        features = f_common.combine_features(features_list=[features_lags, features])

        features, targets, all = f_common.align_features_and_targets(features=features, targets=targets)
        return features, targets, all



from .feature_manager import *

if __name__ == "__main__":
    # calc normal features
    # run_normal_features()
    pass
    # calc periodic features
    # TODO: long running, in case really want it uncomment
    # run_periodic_features()



# from .feature_calculator import *
from .constants import *
import pandas as pd
from .tech_indicators import *


def get_lag_col_name(feature_col: str, lag_len: int):
    return f"Lag-{feature_col}-T-{lag_len:{DECIMAL_PLACE}}"


def get_forward_col_name(feature_col: str, fwd_len: int):
    return f"{feature_col}-Fwd_{fwd_len}"


# todo: all these need to be imported and automated
OPERATORS_IND = {RsiOperator.__name__: RsiOperator,
                 BollingerBandOperator.__name__: BollingerBandOperator,
                 AtrOperator.__name__: AtrOperator,
                 MaOperator.__name__: MaOperator,
                 VilliamrOperator.__name__: VilliamrOperator,
                 MomentomOperator.__name__: MomentomOperator,
                 WmaOperator.__name__: WmaOperator,
                 EmaOperator.__name__: EmaOperator,
                 AdxOperator.__name__: AdxOperator,
                 BopOperator.__name__: BopOperator,
                 HistoricalReturnOperator.__name__: HistoricalReturnOperator
                 # todo: not sure
                 # CandleStickOperators.__name__: CandleStickOperators
                 }

OPERATORS_RET = {HistoricalReturnOperator.__name__: HistoricalReturnOperator}

OPERATORS_FWD = {
                 ForwardMaxPriceAndReturn.__name__: ForwardMaxPriceAndReturn,
                 ForwardMinPriceAndReturn.__name__: ForwardMinPriceAndReturn,
                 ForwardHighestReturn.__name__: ForwardHighestReturn,
                 # AmplitudeBasedLabelerOperator.__name__: AmplitudeBasedLabelerOperator
                 }

def get_operator(operator_name: str):
    if operator_name in OPERATORS_IND.keys():
        col_prefix = RESULT_TECH_IND_PREFIX
        operator = OPERATORS_IND[operator_name]

    elif operator_name in OPERATORS_FWD.keys():
        col_prefix = RESULT_TECH_FWD_PREFIX
        operator = OPERATORS_FWD[operator_name]
    else:
        col_prefix = None
        operator = None

    return operator, col_prefix

class Context:
    def validate(self, data: pd.DataFrame, operators: list, periods: list, lag_periods: list = None,
                 forward_periods: list = None) -> bool:
        try:
            _ = self.get(data=data, operators=operators, periods=periods, lag_periods=lag_periods,
                         forward_periods=forward_periods)
        except Exception as e:
            print(e)
            return False

        return True

    def get(self, data: pd.DataFrame, operators: list, periods: list, lag_periods: list = None,
            forward_periods: list = None) -> pd.DataFrame:
        columns = []
        for operator in operators:
            for period in periods:
                op_columns = self.get_column_names(operator=operator, period=period, lag_periods=lag_periods,
                                               forward_periods=forward_periods)
                if set(op_columns) > set(data.columns):
                    raise Exception(f"invalid column name: operator: {operator.__name__}, period: {period}")

                columns = columns + op_columns
        result = data[columns]
        return result

    def get_column_names(self,
                         period: int,
                         operator: IndicatorOperator = None,
                         operator_name: str = None,
                         lag_periods: list = None,
                         forward_periods: list = None) -> str:

        assert lag_periods is None or forward_periods is None, "cant calculate both forwards and lags at the same time"
        columns = []

        if operator_name is None:
            operator_name = operator.__name__

        operator, col_prefix = get_operator(operator_name=operator_name)
        if operator == None:
            return None

        op_col_names = operator.get_column_names()
        for op_col_name in op_col_names:
            full_name = operator.calculate_final_column_name(prefix=col_prefix, col_name=op_col_name, period=period)
            if lag_periods:
                for lag_period in lag_periods:
                    full_name_lag = get_lag_col_name(feature_col=full_name, lag_len=lag_period)
                    columns.append(full_name_lag)
            elif forward_periods:
                for frwd_period in forward_periods:
                    full_name_fwd = get_forward_col_name(feature_col=full_name, fwd_len=frwd_period)
                    columns.append(full_name_fwd)
            else:
                columns.append(full_name)

            # columns.append(f"{col_prefix}-{op_col_name}-lag_{period}")

        return columns



import pandas as pd
import numpy as np
import investpy
from .data_access import *
from .feature_extractor_news import *


months = np.arange(1, 13)
# years =  [2015, 2016, 2017, 2018, 2019, 2020, 2021]
IMPORTANCE = "high"
# download it for a month, year, day
# download if it is not there or overwrite existing
# flush state
# clean data and store

class NewsFeatureManager():
    def __init__(self, dal: IDataAccessLayer, data_provider_tz: str, countries: list):
        self.dal = dal
        self.data_provider_tz = data_provider_tz
        self.countries = countries


    def _save_month(self, month_n: int, year: int):
        from_date = f"01/{month_n:02d}/{year}"
        end_date = f"01/{month_n + 1:02d}/{year}" if month_n < 12 else f"01/{1:02d}/{year + 1}"
        data = investpy.economic_calendar(time_zone=self.data_provider_tz,
                                          from_date=from_date,
                                          to_date=end_date,
                                          countries=self.countries,
                                          importances=[IMPORTANCE])

        return data

    def save_news_years(self, years):
        for year in years:
            for month_n in months:
                month = f"{month_n:02d}"
                print(f"start processing -  month: {month}, year: {year}")
                data = self._save_month(month_n=month_n, year=year)
                # save
                print("saving news data ==============================")
                self.dal.save_news_data(data, month=month, year=year, if_raw=True)

    def save_news_months(self, months:list, year:int):
        for month in months:
            data = self._save_month(month_n=month, year=year)
            self.dal.save_news_data(news_df=data, month=month, year=year, if_raw=True)

    def get_news(self):
        news_data = self.dal.get_news_data_all(if_raw=True)
        cleaned = clean_data(news_data=news_data)
        return cleaned

    def get_news_month(self, month: int, year: int):
        news_data = self.dal.get_news_data(month=month, year=year, if_raw=True, if_format=True)
        cleaned = clean_data(news_data=news_data)
        return cleaned

# def all_together():
#     download_news()
#     DATA_PROVIDER = "investingcom"
#     dal = H5DataAccessLayer(data_provider=DATA_PROVIDER, resolution="random", symbol="random")
#
#     clean_data["event"] = clean_data["currency"].str.lower() + "_" + clean_data["event"]
